CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
129 lines
4.4 KiB
C++
129 lines
4.4 KiB
C++
/*
|
|
* Copyright (c) Advanced Micro Devices, Inc., or its affiliates.
|
|
*
|
|
* SPDX-License-Identifier: MIT
|
|
*/
|
|
|
|
#include <hip_test_kernels.hh>
|
|
#include <hip_test_common.hh>
|
|
#include <hip_test_checkers.hh>
|
|
#include <complex>
|
|
|
|
#pragma clang diagnostic ignored "-Wunused-variable"
|
|
// Tolerance for error
|
|
const double tolerance = 1e-6;
|
|
|
|
#define LEN 64
|
|
|
|
#define ALL_FUN \
|
|
OP(add) \
|
|
OP(sub) \
|
|
OP(mul) \
|
|
OP(div) \
|
|
OP(abs) \
|
|
OP(arg) \
|
|
OP(sin) \
|
|
OP(cos)
|
|
|
|
#define OP(x) CK_##x,
|
|
enum CalcKind { ALL_FUN };
|
|
#undef OP
|
|
|
|
#define OP(x) \
|
|
case CK_##x: \
|
|
return #x;
|
|
std::string getName(enum CalcKind CK) {
|
|
switch (CK) { ALL_FUN }
|
|
return ""; // To prevent compile warning
|
|
}
|
|
#undef OP
|
|
|
|
// Calculates function.
|
|
// If the function has one argument, B is ignored.
|
|
// If the function returns real number, converts it to a complex number.
|
|
#define ONE_ARG(func) \
|
|
case CK_##func: \
|
|
return std::complex<FloatT>(func(A));
|
|
|
|
template <typename FloatT> __device__ __host__ std::complex<FloatT> calc(std::complex<FloatT> A,
|
|
std::complex<FloatT> B,
|
|
enum CalcKind CK) {
|
|
switch (CK) {
|
|
case CK_add:
|
|
return A + B;
|
|
case CK_sub:
|
|
return A - B;
|
|
case CK_mul:
|
|
return A * B;
|
|
case CK_div:
|
|
return A / B;
|
|
|
|
ONE_ARG(abs)
|
|
ONE_ARG(arg)
|
|
ONE_ARG(sin)
|
|
ONE_ARG(cos)
|
|
}
|
|
return A; // To prevent compile warning
|
|
}
|
|
|
|
template <typename FloatT> __global__ void kernel(std::complex<FloatT>* A, std::complex<FloatT>* B,
|
|
std::complex<FloatT>* C, enum CalcKind CK) {
|
|
int tx = threadIdx.x + blockIdx.x * blockDim.x;
|
|
C[tx] = calc<FloatT>(A[tx], B[tx], CK);
|
|
}
|
|
|
|
template <typename FloatT> void test() {
|
|
typedef std::complex<FloatT> ComplexT;
|
|
|
|
ComplexT *A, *Ad, *B, *Bd, *C, *Cd, *D;
|
|
A = new ComplexT[LEN];
|
|
B = new ComplexT[LEN];
|
|
C = new ComplexT[LEN];
|
|
D = new ComplexT[LEN];
|
|
HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&Ad), sizeof(ComplexT) * LEN));
|
|
HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&Bd), sizeof(ComplexT) * LEN));
|
|
HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&Cd), sizeof(ComplexT) * LEN));
|
|
|
|
for (uint32_t i = 0; i < LEN; i++) {
|
|
A[i] = ComplexT((i + 1) * 1.0f, (i + 2) * 1.0f);
|
|
B[i] = A[i];
|
|
C[i] = A[i];
|
|
}
|
|
HIP_CHECK(hipMemcpy(Ad, A, sizeof(ComplexT) * LEN, hipMemcpyHostToDevice));
|
|
HIP_CHECK(hipMemcpy(Bd, B, sizeof(ComplexT) * LEN, hipMemcpyHostToDevice));
|
|
|
|
// Run kernel for a calculation kind and verify by comparing with host
|
|
// calculation result. Returns false if fails.
|
|
auto test_fun = [&](enum CalcKind CK) {
|
|
hipLaunchKernelGGL(kernel<FloatT>, dim3(1), dim3(LEN), 0, 0, Ad, Bd, Cd, CK);
|
|
HIP_CHECK(hipMemcpy(C, Cd, sizeof(ComplexT) * LEN, hipMemcpyDeviceToHost));
|
|
bool pass = true;
|
|
for (int i = 0; i < LEN; i++) {
|
|
ComplexT Expected = calc(A[i], B[i], CK);
|
|
FloatT error = abs(C[i] - Expected);
|
|
if (abs(Expected) > tolerance) error /= abs(Expected);
|
|
pass &= error < tolerance;
|
|
}
|
|
return pass;
|
|
};
|
|
|
|
#define OP(x) assert(test_fun(CK_##x));
|
|
ALL_FUN
|
|
#undef OP
|
|
|
|
HIP_CHECK(hipFree(Ad));
|
|
HIP_CHECK(hipFree(Bd));
|
|
HIP_CHECK(hipFree(Cd));
|
|
delete[] A;
|
|
delete[] B;
|
|
delete[] C;
|
|
delete[] D;
|
|
}
|
|
|
|
#if HT_AMD
|
|
HIP_TEST_CASE(Unit_StdComplex) {
|
|
SECTION("Test run with float") { test<float>(); }
|
|
SECTION("Test run with double") { test<double>(); }
|
|
}
|
|
#endif
|